require "import"
import "android.widget.*"
import "android.view.*"
import "android.os.*"
import "android.speech.tts.TextToSpeech"
import "java.util.*"

local activity = activity
local tts
local correctAnswers = 0
local wrongAnswers = 0

-- Initializing the TextToSpeech engine
tts = TextToSpeech(activity, TextToSpeech.OnInitListener {
    onInit = function(status)
        if status == TextToSpeech.SUCCESS then
            tts.setLanguage(Locale.US)
        else
            Toast.makeText(activity, "TextToSpeech not available", Toast.LENGTH_SHORT).show()
        end
    end
})

-- Function to generate random math questions
function generateQuestion()
    local random = math.random
    local operator = {"+", "-", "*", "/"}
    local operand1 = random(1, 10)
    local operand2 = random(1, 10)
    local op = operator[random(1, 4)]
    local answer

    if op == "+" then
        answer = operand1 + operand2
    elseif op == "-" then
        answer = operand1 - operand2
    elseif op == "*" then
        answer = operand1 * operand2
    elseif op == "/" then
        operand2 = random(1, operand1) -- Ensure divisibility
        answer = operand1 / operand2
        answer = math.floor(answer * 10) * 0.1 -- Round answer to 1 decimal point to simplify
    end

    return operand1, operand2, op, answer
end

-- Function to speak text
function speakText(text)
    if tts then
        tts.speak(text, TextToSpeech.QUEUE_FLUSH, nil, nil)
    end
end

-- Function to display the question and options
function displayQuestion()
    local questionLayout = LinearLayout(activity)
    questionLayout.setOrientation(LinearLayout.VERTICAL)
    
    local operand1, operand2, op, answer = generateQuestion()
    local questionText = TextView(activity)
    questionText.setText(operand1.." "..op.." "..operand2.." = ?")
    questionText.setTextSize(24)
    questionText.setGravity(Gravity.CENTER)
    questionLayout.addView(questionText)

    local options = {}
    local answers = {answer}

    -- Generating 3 random answers
    for i=1,3 do
        local option
        repeat
            option = math.random(answer - 10, answer + 10)
        until not answers[option]
        answers[option] = true
        table.insert(options, option)
    end

    -- Inserting the correct answer
    table.insert(options, math.random(1, #options + 1), answer)
    
    -- Creating option buttons
    for i, option in ipairs(options) do
        local optionButton = Button(activity)
        optionButton.setText(tostring(option))
        optionButton.setTextSize(20)
        
        -- Answer check
        optionButton.onClick = function(view)
            if option == answer then
                speakText("Correct answer!")
                correctAnswers = correctAnswers + 1
            else
                speakText("oops, Incorrect answer!")
                wrongAnswers = wrongAnswers + 1
            end
            questionLayout.removeAllViews()
            displayQuestion()
        end
        questionLayout.addView(optionButton)
    end

    -- Displaying scores
    local scoreLayout = LinearLayout(activity)
    scoreLayout.setOrientation(LinearLayout.HORIZONTAL)

    local correctText = TextView(activity)
    correctText.setText("Correct answers: " .. correctAnswers)
    scoreLayout.addView(correctText)

    local wrongText = TextView(activity)
    wrongText.setText("Wrong answers: " .. wrongAnswers)
    scoreLayout.addView(wrongText)

    local totalScoreText = TextView(activity)
    totalScoreText.setText("Total Score: " .. (correctAnswers - wrongAnswers))
    scoreLayout.addView(totalScoreText)

    questionLayout.addView(scoreLayout)

    activity.setContentView(questionLayout)
end

displayQuestion()

-- Cleaning up the TTS resources when the activity is destroyed
function onDestroy()
    if tts then
        tts.stop()
        tts.shutdown()
    end
end